from idorudraw import beginLinesEntity, line
from idorudraw import line_from, line_to
import math; M = math

from shapes import circle

### TODO: allow precision in all arcs

### Python 2.3 has it already on math.radians(x)
def radians(x):
    return (x*M.pi)/180.0

def avg(x, y):
    return ((x+y)/2.0)

def pp_distance(x0, y0, x1, y1):
    return M.sqrt((x1-x0)**2 + (y1-y0)**2)

def ll_intersection(m0, x0, y0, m1, x1, y1):
    """
    Takes two [m, x, y] lines, returns a [x, y] point.
    """
    c0 = y0 - (m0*x0)
    c1 = y1 - (m1*x1)
    rx = (c1 - c0)/(m0 - m1)
    ry = (m1*rx) + c1
    return [rx, ry]

def craa(center_x, center_y, ratio, ang0, ang1):
    """
    Draws an arc in the DXF notation, taking center, ratio,
    beginning angle and end angle
    Angles are in degrees
    """
    if M.fabs(ratio):
        p0_x = center_x + M.cos(radians(ang0))*ratio
        p0_y = center_y + M.sin(radians(ang0))*ratio
        p1_x = center_x + M.cos(radians(ang1))*ratio
        p1_y = center_y + M.sin(radians(ang1))*ratio    
        center2(center_x, center_y, p0_x, p0_y, p1_x, p1_y, radios=0)

## FIXME: code duplication
def center2(center_x, center_y, x0, y0, x1, y1, winding=1, radios=1):
    """
    Draws an arc given a center and two points, and
    a direction. The first point determines the radio length.
    """

    q = 12
    
    radio = pp_distance(center_x, center_y, x0, y0)

    angle0=M.atan2(y0-center_y, x0-center_x)*(180.0/M.pi)
    if(angle0 < 0):
        angle0 += 360.0
    angle1=M.atan2(y1-center_y, x1-center_x)*(180.0/M.pi)
    if(angle1 < 0):
        angle1 += 360.0

    ## Normalize point1 to make the radio constant with point0
    radio1 = pp_distance(center_x, center_y, x1, y1)
    factor = radio/radio1
    xn1 = center_x + ((x1-center_x)*factor)
    yn1 = center_y + ((y1-center_y)*factor)

    ### Check the ranges
    if(winding):
        while (angle1 < angle0):
            angle1 += 360.0
        line_from(x0, y0)
        for i in range(int(angle0)+1, int(angle1)-1, q):
            line_to(center_x + M.cos(((float(i)*(M.pi/180))))*radio,
                    center_y + M.sin(((float(i)*(M.pi/180))))*radio)
        line_to(xn1, yn1)
    else:
        while (angle1 > angle0):
            angle1 -= 360.0
        line_from(xn1, yn1)
        ## The 10 is hardcoded: fonts look fine and performance is good
        for i in range(int(angle1)+1, int(angle0)-1, q):
            line_to(center_x + M.cos(((float(i)*(M.pi/180))))*radio,
                    center_y + M.sin(((float(i)*(M.pi/180))))*radio)
        line_to(x0, y0)
        
    if(radios):
        beginLinesEntity()
        line(center_x, center_y, x0, y0)
        line(center_x, center_y, x1, y1)

def points3(x0=10, y0=0, x1=7, y1=7, x2=0, y2=10, radios=0):
    """
    Draws a semicircle from [x0, y0] to [x2, y2]
    passing through [x1, y1] (2D)
    """
    ### Clumsy way to avoid zero divides
    if(M.fabs(y1-y0) < 0.00001):
        y1 += 0.00001
    if(M.fabs(x1-x0) < 0.00001):
        x1 += 0.00001
        
    ## Middle points to hold the arrows
    mid0 = [avg(x0, x1), avg(y0, y1)]
    mid1 = [avg(x1, x2), avg(y1, y2)]
    ## Arrow vectors tangents
    tan0= -((x1-x0)/(y1-y0))
    tan1= -((x2-x1)/(y2-y1))

    center = ll_intersection(tan0, mid0[0], mid0[1],
                             tan1, mid1[0], mid1[1])

    radio = pp_distance(center[0], center[1], x0, y0)
    ## circle(radio, center[0], center[1], q=6)

    ### Draw the arc. Ordering dance...
    angle0=M.atan2(y0-center[1], x0-center[0])*(180.0/M.pi)
    if(angle0 < 0):
        angle0 += 360.0
    angle1=M.atan2(y1-center[1], x1-center[0])*(180.0/M.pi)
    if(angle1 < 0):
        angle1 += 360.0
    angle2=M.atan2(y2-center[1], x2-center[0])*(180.0/M.pi)
    if(angle2 < 0):
        angle2 += 360.0
    adiff0 = angle2 - angle0
    if(adiff0 < 0):
        adiff0 += 360.0
    adiff1 = angle2 - angle1
    if(adiff1 < 0):
        adiff1 += 360.0

    winding = (adiff0 > adiff1)

    # No quality modifier is applied: optimize
    if(winding):
        while (angle1 < angle0):
            angle1 += 360.0
        while(angle2 < angle1):
            angle2 += 360.0
        line_from(x0, y0)
        for i in range(int(angle0)+1, int(angle2)-1):
            line_to(center[0] + M.cos(((float(i)*(M.pi/180))))*radio,
                    center[1] + M.sin(((float(i)*(M.pi/180))))*radio)
        line_to(x2, y2)
    else:
        while (angle1 > angle0):
            angle1 -= 360.0
        while(angle2 > angle1):
            angle2 -= 360.0
        line_from(x2, y2)
        for i in range(int(angle2)+1, int(angle0)-1):
            line_to(center[0] + M.cos(((float(i)*(M.pi/180))))*radio,
                    center[1] + M.sin(((float(i)*(M.pi/180))))*radio)
        line_to(x0, y0)
        
    if(radios):
        beginLinesEntity()
        line(center[0], center[1], x0, y0)
        line(center[0], center[1], x1, y1)
        line(center[0], center[1], x2, y2)
