import idorudraw as D
import arc
import shapes
import cxf_read

def read(file):
    try:
        fp = open(file)
    except IOError:
        print "Error loading"
        return
    draw(fp)
    fp.close()
    return

def draw(fp):

    ## Lines
    D.beginLinesEntity()
    xl = extract_lines(fp)
    for i in range(len(xl)):
        D.line(xl[i][0][0], xl[i][0][1], xl[i][1][0], xl[i][1][1],
               xl[i][0][2], xl[i][1][2])

    ## Arcs
    xa = extract_arcs(fp)
    for i in range(len(xa)):
        arc.craa(xa[i][0], xa[i][1], xa[i][2], xa[i][3], xa[i][4])

    ## Circles
    xc = extract_circles(fp)
    if len(xc) > 0:
        for i in range(len(xc)):
            shapes.circle(xc[i][2], xc[i][0], xc[i][1])

    ## Polylines (R14 only)
    xpl = extract_lwplines(fp)
    if len(xpl) > 0:
        for i in range(len(xpl)):
            D.line_from(xpl[i][0][0], xpl[i][0][1])
            for j in range(0, len(xpl[i][1]), 2):
                D.line_to(xpl[i][1][j], xpl[i][1][j+1])

    ## Text (Doesn't handle fonts with ARs, doesn't handle font style,
    ## doesn't handle font separation at all)
    txt = extract_texts(fp)
    ## Harcoded: FIXME
    fon = cxf_read.load_font("cursive")
    for i in range(len(txt)):
        cxf_read.write(txt[i][1], fon,
                       txt[i][0][0], txt[i][0][1],
                       txt[i][2], txt[i][3])

def group(file):
    line = file.readline()
    try:
        id = int(line)
    except ValueError:
        print "Bad file structure"
        raise
        return
    str = file.readline() ## fixme: check str for non-whitespace chars
    return [id, str]

def is_dxf(fp):
    fp.seek(0)
    id = fp.readline(); str = fp.readline();
    ## if id == 0 and ... [FIXME]
    if True and str.find("SECTION") != -1:
        return True
    else:
        return False

def group_is(id, str, lid, lstr):
    return ((id == lid) and (str.find(lstr) == 0))

def extract_lines(fp):
    fp.seek(0)

    x1 = 0.0; y1 = 0.0; z1 = 0.0
    x2 = 0.0; y2 = 0.0; z2 = 0.0    
    id = -1; val = ""

    res = []

    id, val = group(fp)

    ## File loop
    while True:

        if group_is(id, val, 0, "EOF"):
            break

        if group_is(id, val, 2, "ENTITIES"):

            ## Entity loop
            while True:
                if group_is(id, val, 0, "ENDSEC"):
                    break
                if group_is(id, val, 0, "LINE"):
                    id, val = group(fp)

                    ## Line loop
                    while True:
                        if id == 10:
                            x1 = float(val)
                        elif id == 20:
                            y1 = float(val)
                        elif id == 30:
                            z1 = float(val)
                        elif id == 11:
                            x2 = float(val)
                        elif id == 21:
                            y2 = float(val)
                        elif id == 31:
                            z2 = float(val)
                        id, val = group(fp)
                        if id == 0:
                            break
                    res.extend([[[x1, y1, z1],[x2, y2, z2]]])
                else:
                    id, val = group(fp)
        else:
            id, val = group(fp)

    return res

def extract_arcs(fp):
    fp.seek(0)

    x1 = 0.0; y1 = 0.0;
    ratio = 0.0; a1 = 0.0; a2 = 0.0    
    id = -1; val = ""

    res = []

    id, val = group(fp)

    ## File loop
    while True:

        if group_is(id, val, 0, "EOF"):
            break

        if group_is(id, val, 2, "ENTITIES"):

            ## Entity loop
            while True:
                if group_is(id, val, 0, "ENDSEC"):
                    break
                if group_is(id, val, 0, "ARC"):
                    id, val = group(fp)

                    ## Arc loop
                    while True:
                        if id == 10:
                            x1 = float(val)
                        elif id == 20:
                            y1 = float(val)
                        elif id == 30:
                            z1 = float(val)
                        elif id == 40:
                            ratio = float(val)
                        elif id == 50:
                            a1 = float(val)
                        elif id == 51:
                            a2 = float(val)
                        id, val = group(fp)
                        if id == 0:
                            break
                    res.extend([[x1, y1, ratio, a1, a2]])
                else:
                    id, val = group(fp)
        else:
            id, val = group(fp)

    return res

def extract_circles(fp):
    fp.seek(0)

    x1 = 0.0; y1 = 0.0;
    ratio = 0.0; a1 = 0.0; a2 = 0.0    
    id = -1; val = ""

    res = []

    id, val = group(fp)

    ## File loop
    while True:

        if group_is(id, val, 0, "EOF"):
            break

        if group_is(id, val, 2, "ENTITIES"):

            ## Entity loop
            while True:
                if group_is(id, val, 0, "ENDSEC"):
                    break
                if group_is(id, val, 0, "CIRCLE"):
                    id, val = group(fp)

                    ## Arc loop
                    while True:
                        if id == 10:
                            x1 = float(val)
                        elif id == 20:
                            y1 = float(val)
                        elif id == 30:
                            z1 = float(val)
                        elif id == 40:
                            ratio = float(val)
                        id, val = group(fp)
                        if id == 0:
                            break
                    res.extend([[x1, y1, ratio]])
                else:
                    id, val = group(fp)
        else:
            id, val = group(fp)

    return res

## Apparently only 2D? Check later
## These are the most common polylines in wilderness
## Works in a saved file as R14 - but the R13 uses POLYLINE instead
def extract_lwplines(fp):
    fp.seek(0)

    x = 0.0; y = 0.0; z = 0.0
    length = 0
    id = -1; val = ""

    ret = []
    nibble = []; pline = []
    id, val = group(fp)

    ## File loop
    while True:
        if group_is(id, val, 0, "EOF"):
            break

        if group_is(id, val, 2, "ENTITIES"):

            ## Entity loop
            while True:

                if group_is(id, val, 0, "ENDSEC"):
                    break
                
                if group_is(id, val, 0, "LWPOLYLINE"):
                    id, val = group(fp)
                    
                    ## Polyline loop
                    while True:
                        if id == 90:
                            length = int(val)
                        elif id == 10:
                            ## let's assume it's 2D and the values come as
                            ## 10, 20, 10, 20... until length wears out
                            ## Fixme: add sanity checks

                            ## Line from:
                            nibble = []
                            nibble.append(float(val))
                            id, val = group(fp)
                            nibble.append(float(val))

                            ## Line to:
                            pline = []
                            for i in range(0, (length-1)*2):
                                id, val = group(fp)
                                pline.append(float(val))
                                
                        id, val = group(fp)
                        if id == 0:
                            break
                    ret.append([nibble,pline])
                else:
                    id, val = group(fp)
        else:
            id, val = group(fp)
    return ret
        
    
def extract_texts(fp):
    fp.seek(0)

    x = 0.0; y = 0.0
    txt = ""; height = 1; align = 0
    id = -1; val = ""

    ret = []

    id, val = group(fp)

    ## File loop
    while True:
        if group_is(id, val, 0, "EOF"):
            break

        if group_is(id, val, 2, "ENTITIES"):

            ## Entity loop
            while True:

                if group_is(id, val, 0, "ENDSEC"):
                    break
                
                if group_is(id, val, 0, "TEXT"):
                    id, val = group(fp)
                    
                    ## Polyline loop
                    while True:
                        if id == 10:
                            x = float(val)
                        elif id == 20:
                            y = float(val)
                        elif id == 40:
                            height = float(val)
                        elif id == 1:
                            txt = val
                        elif id == 72:
                            align = int(val)
                            
                        id, val = group(fp)
                        if id == 0:
                            break
                    ret.append([[x,y],txt, height, align])
                else:
                    id, val = group(fp)
        else:
            id, val = group(fp)
    return ret
