import idorudraw as D
import math as M
import cadget

def lathe(axis, offset, list):
    """
    'axis' should be a (xyz) tuple with center on the origin
    'offset' is another (xyz) tuple displacing the center
    'list' is the tuple returned by cadget.getentity()
    """

    ## list[0] is the type of the entity. 1 is LINE, 3 is LINE_STRIP
    if list[0] == 1 or list[0] == 3:
        for i in range(0, len(list[1])):
            if list[0] == 3:
                line = (list[1][i], list[1][i+1])
                belt(axis, offset, line)
            else:
                if not i%2:
                    line = (list[1][i], list[1][i+1])
                    belt(axis, offset, line)

def belt(axis, offset, list):
    """
    'axis' should be a (xyz) tuple centered in the origin
    'offset' is another (xyz) tuple displacing the center
    'list' is a ((xyz),(xyz)) two points line tuple
    The behavior of this function is to draw a triangle strip around 'axis'
    """

    D.triangle_from(list[1][0], list[1][1], list[0][0], list[0][1],
                    list[1][2], list[0][2])

    for i in range(0, 361, 12):
        x1, y1, z1 = axis_rot((offset, axis), list[0], i)
        x2, y2, z2 = axis_rot((offset, axis), list[1], i)
        D.triangle_to(x1, y1, z1)
        D.triangle_to(x2, y2, z2)

def axis_rot(axis, p, angle):
    """
    'axis' is a ((xyz)(xyz)) tuple where the first value is the
    displacement from the origin and the second a normalized  vector from it.
    'point' is the (xyz) tuple to transform
    'angle' is an angle in degrees
    Function returns a (xyz) tuple: the transformation of 'point' around
    'axis' by 'angle' degrees.
    """

    c = M.cos(float(angle)*M.pi/180.0)
    s = M.sin(float(angle)*M.pi/180.0)
    cc = 1.0-c

    point = (p[0] - axis[0][0], p[1] - axis[0][1], p[2] - axis[0][2])

    ## The following calculations are the multiplication of a matrix chain
    ## Reference: http://astronomy.swin.edu.au/~pbourke/geometry/rotate/
    
    x = ((point[0] *
          (c + cc * axis[1][0] * axis[1][0])) + 
         (point[1] *
          (cc * axis[1][0] * axis[1][1] - axis[1][2] * s)) +
         (point[2] *
          (cc * axis[1][0] * axis[1][2] + axis[1][1] * s)))

    y = ((point[0] *
          (cc * axis[1][1] * axis[1][0] + axis[1][2] * s)) +
         (point[1] *
          (c + cc * axis[1][1] * axis[1][1])) +
         (point[2] *
          (cc * axis[1][1] * axis[1][2] - axis[1][0] * s)))

    z = ((point[0] *
          (cc * axis[1][2] * axis[1][0] - axis[1][1] * s)) +
         (point[1] *
          (cc * axis[1][2] * axis[1][1] + axis[1][0] * s)) +
         (point[2] *
          (c + cc * axis[1][2] * axis[1][2])))
    
    return (x + axis[0][0], y  + axis[0][1], z  + axis[0][2])
